define([
        'freeipa/builder',
        'freeipa/_base/Spec_mod',
        'freeipa/ipa',
        'freeipa/phases',
        'freeipa/menu',
        'freeipa/reg',
        'freeipa/user',
        'freeipa/group',
        'freeipa/host',
        'freeipa/widget',
        'freeipa/dialog'
    ],
    function (builder, SpecMod, IPA, phases, menu, reg, user, group, host, dialogs) {
        let mail_server = {};

        /**
         * Entity spec
         */
var make_mailalias_spec = function() {
return {
    name: 'mailalias',
    facets: [
        {
            $type: 'search',
            label: 'Aliases',
            custom_actions: [
                {
                    $type: 'add',
                    hide_cond: []
                },
                {
                    $type: 'batch_remove',
                    hide_cond: []
                }
            ],
            show_values_with_dup_key: true,
            columns: [
                'imsmailaliassource',
                'imsmailaliastargets'
            ]
        },
        {
            $type: 'details',
            sections: [
                {
                    name: 'global_info',
                    layout_ccs_class: 'col-sm-12',
                    fields: [
                        {
                            name: 'imsmailaliassource'
                        },
                        {
                            $type: 'multivalued',
                            name: 'imsmailaliastargets'
                        }
                   ]
                }
            ]
        }
    ],
    adder_dialog: {
        title: 'Mail alias add dialog',
        fields: [
            {
                name: 'imsmailaliassource',
                required: true
            },
            {
                name: 'imsmailaliastargets',
                required: true
            }
        ],
        height: 300
    },
    deleter_dialog: {
        title: 'Mail alias delete dialog'
    }
};};

var make_mailbox_spec = function() {
return {
    name: 'mailbox',
    facets: [
        {
            $type: 'search',
            label: 'Mailboxes',
            custom_actions: [
                {
                    $type: 'add',
                    hide_cond: []
                },
                {
                    $type: 'batch_remove',
                    hide_cond: []
                }
            ],
            show_values_with_dup_key: true,
            columns: [
                'imsmailaddress',
                'description'
            ]
        },
        {
            $type: 'details',
            name: 'mailbox',
            sections: [
                {
                    name: 'global_info',
                    layout_ccs_class: 'col-sm-12',
                    fields: [
                        'imsmailaddress',
                        'description',
                        'imsmailboxclassdn',
                        'imsmailboxquota',
                        'imsmailboxtransport'
                   ]
                },
                {
                    $factory: IPA.section,
                    name: 'divider',
                    layout_css_class: 'col-sm-12',
                    fields: []
                },
                {
                    $factory: IPA.section,
                    name: 'members',
                    label: '@i18n:objects.vault.members',
                    fields: [
                        {
                            $type: 'association_table',
                            id: 'imsmailboxmember_user_cn',
                            name: 'imsmailboxmember_user',
                            acl_param: 'imsmailboxmember',
                            columns: [
                                {
                                    name: 'imsmailboxmember_user',
                                    label: 'Member users'
                                }
                            ],
                            add_title: '@i18n:objects.group.add_users',
                            remove_title: '@i18n:objects.group.remove_users'
                        },
                        {
                            $type: 'association_table',
                            id: 'imsmailboxmember_group_cn',
                            name: 'imsmailboxmember_group',
                            other_entity: 'group',
                            acl_param: 'imsmailboxmember',
                            columns: [
                                {
                                    name: 'imsmailboxmember_group',
                                    label: 'Member groups'
                                }
                            ],
                            add_title: '@i18n:objects.group.add_groups',
                            remove_title: '@i18n:objects.group.remove_groups'
                       }
                   ]
               }
            ]
        }
    ],
    adder_dialog: {
        title: 'Mailbox add dialog',
        fields: [
            {
                name: 'imsmailaddress',
                required: true
            },
            {
                name: 'description',
                required: false
            }
        ],
        height: 300
    },
    deleter_dialog: {
        title: 'Mailbox delete dialog'
    }
};};

var make_mailboxclass_spec = function() {
return {
    name: 'mailboxclass',
    facets: [
        {
            $type: 'search',
            label: 'Mailbox classes',
            custom_actions: [
                {
                    $type: 'add',
                    hide_cond: []
                },
                {
                    $type: 'batch_remove',
                    hide_cond: []
                }
            ],
            show_values_with_dup_key: true,
            details_facet: 'mailboxclass',
            columns: [
                'cn',
                'description',
                'imsmailboxquota'
            ]
        },
        {
            $type: 'details',
            name: 'mailboxclass',
            sections: [
                {
                    name: 'global_info',
                    layout_ccs_class: 'col-sm-12',
                    fields: [
                        'cn',
                        'description',
                        'imsmailboxquota'
                   ]
                }
            ]
        }
    ],
    adder_dialog: {
        title: 'Mailbox class add dialog',
        fields: [
            {
                name: 'cn',
                required: true
            },
            {
                name: 'description',
                required: false
            },
            {
                name: 'imsmailboxquota',
                required: false
            }
        ],
        height: 300
    },
    deleter_dialog: {
        title: 'Mailbox class delete dialog'
    }
};};


var make_maildomain_spec = function() {
return {
    name: 'maildomain',
    facets: [
        {
            $type: 'search',
            label: 'Mail domains',
            custom_actions: [
                {
                    $type: 'add',
                    hide_cond: []
                },
                {
                    $type: 'batch_remove',
                    hide_cond: []
                }
            ],
            show_values_with_dup_key: true,
            columns: [
                'imsmaildomain',
                'description',
                'imsmailboxdefaultclassdn',
                'imsmailboxdefaultquota',
                'imsmailboxtransport'
            ]
        },
        {
            $type: 'details',
            name: 'maildomain',
            sections: [
                {
                    name: 'global_info',
                    layout_ccs_class: 'col-sm-12',
                    fields: [
                        'imsmaildomain',
                        'description',
                        'imsmailboxdefaultclassdn',
                        'imsmailboxdefaultquota',
                        'imsmailboxtransport'
                   ]
                }
            ]
        }
    ],
    adder_dialog: {
        title: 'Mail domain add dialog',
        fields: [
            {
                name: 'imsmaildomain',
                required: true
            },
            {
                name: 'description',
                required: false
            },
            {
                name: 'imsmailboxtransport',
                required: false
            }
        ],
        height: 300
    },
    deleter_dialog: {
        title: 'Mail domain delete dialog'
    }
};};


        mail_server.mailalias_spec = make_mailalias_spec();
        mail_server.mailbox_spec = make_mailbox_spec();
        mail_server.mailboxclass_spec = make_mailboxclass_spec();
        mail_server.maildomain_spec = make_maildomain_spec();

        mail_server.add_menu = function () {
            if (IPA.is_selfservice === false) {
                menu.add_item({
                        name: 'mailserver',
                        label: 'Mail Server',
                        entity: 'maildomain',
                        facet: 'search',
                        children: [
                            {
                                entity: 'maildomain',
                                facet: 'search',
                                hidden: false
                            },
                            {
                                entity: 'mailboxclass',
                                facet: 'search',
                                hidden: false
                            },
                            {
                                entity: 'mailbox',
                                facet: 'search',
                                hidden: false
                            },
                            {
                                entity: 'mailalias',
                                facet: 'search',
                                hidden: false
                            }
                        ]
                    },
                    'network_services');
            }
        };


        mail_server.register = function () {
            let e = reg.entity;
            let a = reg.action;
            let d = reg.dialog;
            let fa = reg.facet;

            e.register({type: 'mailalias', spec: mail_server.mailalias_spec});
            e.register({type: 'mailbox', spec: mail_server.mailbox_spec});
            e.register({type: 'mailboxclass', spec: mail_server.mailboxclass_spec});
            e.register({type: 'maildomain', spec: mail_server.maildomain_spec});
        };

        phases.on('registration', mail_server.register);
        phases.on('profile', mail_server.add_menu, 20);
        return mail_server;
    });
